#
# Copyright (c) 2008-2018 Thierry Florac <tflorac AT ulthar.net>
# All Rights Reserved.
#
# This software is subject to the provisions of the Zope Public License,
# Version 2.1 (ZPL).  A copy of the ZPL should accompany this distribution.
# THIS SOFTWARE IS PROVIDED "AS IS" AND ANY AND ALL EXPRESS OR IMPLIED
# WARRANTIES ARE DISCLAIMED, INCLUDING, BUT NOT LIMITED TO, THE IMPLIED
# WARRANTIES OF TITLE, MERCHANTABILITY, AGAINST INFRINGEMENT, AND FITNESS
# FOR A PARTICULAR PURPOSE.
#

__docformat__ = 'restructuredtext'

from zope.interface import implementer, provider
from zope.schema.fieldproperty import FieldProperty

from onf_website.component.hearing import IHearingTarget
from onf_website.component.location import ILocationTarget
from onf_website.shared.report.interfaces import IReport, IWfReport, IWfReportFactory, REPORT_CONTENT_NAME, \
    REPORT_CONTENT_TYPE
from pyams_content.component.illustration import ILinkIllustrationTarget
from pyams_content.component.paragraph.interfaces import IParagraphContainerTarget
from pyams_content.component.theme.interfaces import ICollectionsTarget, ITagsTarget, IThemesTarget
from pyams_content.features.preview.interfaces import IPreviewTarget
from pyams_content.features.review.interfaces import IReviewTarget
from pyams_content.shared.common import SharedContent, register_content_type
from pyams_content.shared.common.interfaces import IWfSharedContentFactory
from pyams_content.shared.common.types import WfTypedSharedContent
from pyams_utils.adapter import adapter_config


@implementer(IWfReport, ILinkIllustrationTarget, IParagraphContainerTarget,
             ILocationTarget, IHearingTarget, ITagsTarget, IThemesTarget, ICollectionsTarget,
             IPreviewTarget, IReviewTarget)
class WfReport(WfTypedSharedContent):
    """Report class"""

    content_type = REPORT_CONTENT_TYPE
    content_name = REPORT_CONTENT_NAME

    references = FieldProperty(IWfReport['references'])


register_content_type(WfReport)


@provider(IWfReportFactory)
@implementer(IReport)
class Report(SharedContent):
    """Workflow managed report"""


@adapter_config(context=IWfReportFactory, provides=IWfSharedContentFactory)
def report_content_factory(context):
    return WfReport
